/**
 * @file
 *
 * Navigation bar improvements and tweaks.
 *
 * The biggest addition is support for 3 levels of depth for small screens.
 */

(function ($, drupalSettings) {

  'use strict';

  /**
   * Toggle all dropdown menu items in active path.
   *
   * @param $navbar
   *   Navigation bar jQuery element.
   * @param state
   *   Pass true or false to specifically set the aria-expanded state. If not
   *   used aria-expanded state value is toggled.
   */
  var toggleActivePath = function($navbar, state) {
    $navbar.find('.dropdown > a.is-active').each(function () {
      toggleDropdown($(this), state);
    });
  };

  /**
   * Toggle dropdown for a given dropdown toggle and it parent.
   *
   * @param $dropdownToggle
   * @param state
   *   Pass true or false to specifically set the aria-expanded state. If not
   *   used aria-expanded state value is toggled.
   */
  var toggleDropdown = function($dropdownToggle, state) {
    $($dropdownToggle.parents('.dropdown').get(0)).toggleClass('show');
    // If state is passed use it, otherwise toggle the state.
    var newState = state !== null && typeof(state) !== 'undefined' ? state : ($dropdownToggle.attr('aria-expanded') === 'false' ? 'true' : 'false');
    $dropdownToggle.attr('aria-expanded', newState);
    $dropdownToggle.siblings('.dropdown-menu').toggleClass('show');
  };

  /**
   * Find open dropdown items and close them in a given dropdown parent.
   *
   * @param $dropdownParent
   */
  var closeDropdowns = function($dropdownParent) {
    $dropdownParent.find('> .dropdown.show').each(function () {
      var $this = $(this);
      $this.removeClass('show');
      $this.find('> a').attr('aria-expanded', 'false');
      $this.find('> .dropdown-menu').removeClass('show');
    });
  };

  /**
   * Check is navbar shown.
   *
   * @param $element
   *   Element that belongs to navbar.
   * @return {boolean}
   *   True if navbar is shown, or false if not.
   */
  var isNavbarColapseShow = function($element) {
    // If parent .navbar-collapse has show class this means that we are
    // in small view where responsive navbar is visible.
    var $navbarCollapse = $element.parents('.navbar-collapse');
    // canvas-slid is used for off canvas navigation
    // @todo can we maybe normalize this so show class is used in both cases?
    if ($navbarCollapse.hasClass('show') || $navbarCollapse.hasClass('canvas-slid')) {
      return true;
    }
    return false;
  };

  /**
   * Follow the link.
   *
   * @param $link
   *   jQuery element with href and optional target attribute.
   */
  var followLink = function($link) {
    // Load a page in browser and disable propagation of event.
    if ($link.attr('target')) {
      window.open($link.attr('href'), $link.attr('target'));
    }
    else {
      window.location = $link.attr('href');
    }
  };

  /**
   * Open sub-navigation items on hover.
   */
  function subNavigationOnHover() {
    $('.navbar-nav').each(function () {
      var $navbar = $(this);
      var dropdownItems = [];
      var initialActiveMenu = null;

      function activateMenuItem(item) {
        item.item.addClass('active');
        if (!item.initiallyActive) {
          item.itemLink.addClass('active is-active');
        }
        item.itemLink.attr('aria-expanded', true);
        item.active = true;
      }

      function deactivateMenuItem(item) {
        item.item.removeClass('active');
        if (!item.initiallyActive) {
          item.itemLink.removeClass('active is-active');
        }
        item.itemLink.attr('aria-expanded', false);
        item.active = false;
      }

      $navbar.find('> .nav-item.dropdown').each(function (index) {
        var itemIndex = index;
        var $item = $(this);
        var menuItem = {
          item: $item,
          itemLink: $item.find('> a'),
          active: $item.hasClass('active')
        };
        if (menuItem.active) {
          menuItem.initiallyActive = menuItem.active;
          initialActiveMenu = menuItem;
        }

        dropdownItems.push(menuItem);

        // Hover in.
        var hoverIn = function () {
          if (!menuItem.active) {
            // Remove active class from all other dropdown items.
            for (var i = 0; i < dropdownItems.length; ++i) {
              if (i !== itemIndex) {
                deactivateMenuItem(dropdownItems[i]);
              }
            }

            activateMenuItem(menuItem);
          }
        };

        // Hover out.
        var hoverOut = function () {
          if (menuItem.active && !menuItem.initiallyActive) {
            deactivateMenuItem(menuItem);
            if (initialActiveMenu) {
              activateMenuItem(initialActiveMenu);
            }
          }
        };

        // Use jQuery.hoverIntent plugin if it exist.
        if ($.fn.hoverIntent) {
          // Config option timeout is increased a bit to allow more easier
          // pointer moving from item to submenu.
          $item.hoverIntent({over: hoverIn, out: hoverOut, timeout: 100});
        }
        else {
          $item.hover(hoverIn, hoverOut);
        }
      });
    });
  }

  $(function () {
    if (drupalSettings.bs_bootstrap.navbar_type === 'second-level-horizontal') {
      // Optimize dropdown toggle menu items click event.
      $('.navbar-nav .dropdown > a').click(function (e) {
        var $this = $(this);

        // If menu item is active (expanded) follow the link.
        // This allow us to use expanded dropdown toggle links as regular
        // navigation elements.
        if ($this.attr('aria-expanded') === 'true') {
          followLink($this);
          return false;
        }
      });

      // Open sub-navigation items on hover.
      if (drupalSettings.bs_bootstrap.navbar_onhover) {
        subNavigationOnHover();
      }
    }

    // Toggle active paths.
    $('.navbar-toggler').each(function () {
      var $this = $(this);
      var $navbar = $($this.data('target')).find('.navbar-nav');

      $this.click(function () {
        toggleActivePath($navbar, true);
      });
    });

    // Small screen third level support for dropdown.
    // Taken and fixed from http://stackoverflow.com/a/18682698.
    // @todo - we should support second level the same way as we support third
    // level, but Bootstrap dropdown.js is making problems here (stealing the
    // click and reloading the page probably). This is not ideal and should be
    // improved.
    $('.navbar-nav .dropdown .dropdown > a').each(function () {
      var $this = $(this);
      var $parent = $($this.parents('.dropdown-menu').get(0));

      $this.on('click', function(event) {
        // If we are on big screen then don't continue.
        if (!isNavbarColapseShow($this)) {
          return;
        }

        // Avoid following the href location when clicking.
        event.preventDefault();

        // Avoid having the menu to close when clicking.
        event.stopPropagation();

        // Close first any open dropdown in the same parent.
        // @todo - this is commented for now to avoid user focus losing when
        // we have open long menus and closing them will trigger viewport
        // vertical scrolling.
        //closeDropdowns($parent);

        // Open the one we clicked on.
        toggleDropdown($this);
      });
    });

    // Offcanvas close link support.
    var $offcanvas = $('.offcanvas');
    $('.offcanvas-close-link').click(function () {
      $offcanvas.offcanvas('hide');
    });
  });

})(jQuery, drupalSettings);
